<?php
/**
 * Copyright since 2022 PayGreen
 * NOTICE OF LICENSE
 * This source file is subject to the MIT License X11
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/mit-license.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@paygreen.fr so we can send you a copy immediately.
 * @author    PayGreen <contact@paygreen.fr>
 * @copyright 2014 - 2022 Watt Is It
 * @license   https://opensource.org/licenses/mit-license.php MIT License X11
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class PaygreenPaymentOrderModel extends ObjectModel
{
    /** @var int */
    public $order_id;

    /** @var string */
    public $payment_order_id;

    /** @var int */
    public $created_at;

    /** @var int */
    public $updated_at;

    /**
     * @var array
     * @see ObjectModel::$definition
     */
    public static $definition = [
        'table' => 'paygreen_payment_order',
        'primary' => 'id',
        'fields' => [
            'order_id' => [
                'type' => ObjectModel::TYPE_INT,
                'validate' => 'isInt',
            ],
            'payment_order_id' => [
                'type' => ObjectModel::TYPE_STRING,
                'size' => 255,
            ],
            'created_at' => [
                'type' => ObjectModel::TYPE_INT,
                'validate' => 'isInt',
            ],
            'updated_at' => [
                'type' => ObjectModel::TYPE_INT,
                'validate' => 'isInt',
            ],
        ],
    ];

    /**
     * @return int
     */
    public function getOrderId()
    {
        return $this->order_id;
    }

    /**
     * @param int $order_id
     * @return void
     */
    public function setOrderId($order_id)
    {
        $this->order_id = $order_id;
    }

    /**
     * @return string
     */
    public function getPaymentOrderId()
    {
        return $this->payment_order_id;
    }

    /**
     * @param string $payment_order_id
     *
     * @return void
     */
    public function setPaymentOrderId($payment_order_id)
    {
        $this->payment_order_id = $payment_order_id;
    }

    /**
     * @return int
     */
    public function getCreatedAt()
    {
        return $this->created_at;
    }

    /**
     * @param int $created_at
     * @return void
     */
    public function setCreatedAt($created_at)
    {
        $this->created_at = $created_at;
    }

    /**
     * @return int
     */
    public function getUpdatedAt()
    {
        return $this->updated_at;
    }

    /**
     * @param int $updated_at
     * @return void
     */
    public function setUpdatedAt($updated_at)
    {
        $this->updated_at = $updated_at;
    }

    /**
     * @param int $order_id
     *
     * @return self
     *
     * @throws PrestaShopException
     */
    public function getByOrderId($order_id)
    {
        $query = new DbQuery();
        $query->select('*');
        $query->from(static::$definition['table']);
        $query->where('order_id = ' . (int) $order_id);

        /** @phpstan-ignore-next-line */
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($query->build());

        if (false === $result) {
            return $this;
        }

        // @phpstan-ignore-next-line
        $this->hydrate($result);

        return $this;
    }
}
