<?php
/**
 * Copyright since 2022 PayGreen
 * NOTICE OF LICENSE
 * This source file is subject to the MIT License X11
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/mit-license.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@paygreen.fr so we can send you a copy immediately.
 * @author    PayGreen <contact@paygreen.fr>
 * @copyright 2014 - 2022 Watt Is It
 * @license   https://opensource.org/licenses/mit-license.php MIT License X11
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

require_once __DIR__ . '/../classes/PaygreenPaymentLockModel.php';
require_once __DIR__ . '/PaygreenPaymentUtils.php';

class PaygreenPaymentLockHelper
{
    const LOCK_DURATION = 10;
    const LOCK_WAITING = 2;

    /**
     * @param string $id
     * @param string $context
     * @return void
     * @throws PrestaShopException
     */
    public static function lock($id, $context)
    {
        if (!self::isLocked($id, $context)) {
            try {
                PaygreenPaymentUtils::addLog(strtoupper($context) . ' - PaygreenPaymentLockHelper::lock - ' . $id . ' - Locking process');
                $paygreenLock = new PaygreenPaymentLockModel();
                $paygreenLock->setSubjectId($id);
                $timestamp = (new DateTime())->getTimestamp();
                $paygreenLock->setCreatedAt($timestamp);
                $paygreenLock->setExpiredAt($timestamp + self::LOCK_DURATION);
                $paygreenLock->save();
            } catch (Exception $exception) {
                PaygreenPaymentUtils::addLog(strtoupper($context) . ' - PaygreenPaymentLockHelper::lock - ' . $id . ' - Locking  has failed', 2);
            }
        }
    }

    /**
     * @param string $id
     * @param string $context
     * @return void
     * @throws PrestaShopException
     */
    public static function unlock($id, $context)
    {
        if (self::isLocked($id, $context)) {
            try {
                PaygreenPaymentUtils::addLog(strtoupper($context) . ' - PaygreenPaymentLockHelper::unlock - ' . $id . ' - Unlocking process');
                $paygreenLock = new PaygreenPaymentLockModel();
                $paygreenLock->getBySubjectId($id);
                $paygreenLock->delete();
            } catch (Exception $exception) {
                PaygreenPaymentUtils::addLog(strtoupper($context) . ' - PaygreenPaymentLockHelper::unlock - ' . $id . ' - Unlocking  has failed', 2);
            }
        }
    }

    /**
     * @param string $id
     * @param string $context
     * @return bool
     * @throws PrestaShopException
     */
    public static function isLocked($id, $context)
    {
        $paygreenLock = new PaygreenPaymentLockModel();
        $paygreenLock->getBySubjectId($id);
        $timestamp = (new DateTime())->getTimestamp();

        if (!empty($paygreenLock->getSubjectId() && ($timestamp < $paygreenLock->getExpiredAt()))) {
            PaygreenPaymentUtils::addLog(strtoupper($context) . ' - PaygreenPaymentLockHelper::isLocked - ' . $id . ' - Currently locked');

            return true;
        }

        return false;
    }
}
